"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.authMiddleware = exports.getToken = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _crypto = _interopRequireDefault(require("crypto"));

var _basicAuth = _interopRequireDefault(require("basic-auth"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var getToken = function getToken(apiToken, cookieFile) {
  if (apiToken && cookieFile) throw new Error('Please specify one of --api-token or --cookie-file, not both');

  if (cookieFile) {
    if (_fs["default"].existsSync(cookieFile)) apiToken = _fs["default"].readFileSync(cookieFile).toString();else _fs["default"].writeFileSync(cookieFile, apiToken = _crypto["default"].randomBytes(32).toString('hex'));
  }

  if (!apiToken) throw new Error('Missing access token. Please configure one of --api-token or --cookie-file');
  return apiToken;
};

exports.getToken = getToken;

var authMiddleware = function authMiddleware(name, pass) {
  var realm = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'Lightning Charge';
  return function (req, res, next) {
    var cred = (0, _basicAuth["default"])(req);
    if (!cred || cred.name !== name || cred.pass !== pass) res.set('WWW-Authenticate', "Basic realm=\"".concat(realm, "\"")).removeHeader('Access-Control-Allow-Origin').sendStatus(401);else next();
  };
};

exports.authMiddleware = authMiddleware;