"use strict";

var _nanoid = require("nanoid");

var _exchangeRate = require("./lib/exchange-rate");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var debug = require('debug')('lightning-charge'),
    status = function status(inv) {
  return inv.pay_index ? 'paid' : inv.expires_at > now() ? 'unpaid' : 'expired';
},
    format = function format(inv) {
  return _objectSpread(_objectSpread({}, inv), {}, {
    status: status(inv),
    msatoshi: inv.msatoshi || null,
    metadata: JSON.parse(inv.metadata)
  });
},
    now = function now(_) {
  return Date.now() / 1000 | 0;
}; // @XXX invoices that accept any amount are stored as msatoshi='' (empty string)
// and converted to null when formatted. this is due to sqlite's lack of support
// for ALTER TABLE, which makes dropping the NOT NULL constraint complicated
// (requires creating a new table, copying over the data and replacing the old one).
// This will eventually be done in a future release.


var defaultDesc = process.env.INVOICE_DESC_DEFAULT || 'Lightning Charge Invoice';

module.exports = function (db, ln) {
  var newInvoice = /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(props) {
      var currency, amount, expiry, description, metadata, webhook, id, msatoshi, desc, lninv, invoice;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              currency = props.currency, amount = props.amount, expiry = props.expiry, description = props.description, metadata = props.metadata, webhook = props.webhook;
              id = (0, _nanoid.nanoid)();

              if (!props.msatoshi) {
                _context.next = 6;
                break;
              }

              _context.t0 = '' + props.msatoshi;
              _context.next = 14;
              break;

            case 6:
              if (!currency) {
                _context.next = 12;
                break;
              }

              _context.next = 9;
              return (0, _exchangeRate.toMsat)(currency, amount);

            case 9:
              _context.t1 = _context.sent;
              _context.next = 13;
              break;

            case 12:
              _context.t1 = '';

            case 13:
              _context.t0 = _context.t1;

            case 14:
              msatoshi = _context.t0;
              desc = props.description ? '' + props.description : defaultDesc;
              _context.next = 18;
              return ln.invoice(msatoshi || 'any', id, desc, expiry);

            case 18:
              lninv = _context.sent;
              invoice = {
                id: id,
                msatoshi: msatoshi,
                description: desc,
                quoted_currency: currency,
                quoted_amount: amount,
                rhash: lninv.payment_hash,
                payreq: lninv.bolt11,
                expires_at: lninv.expires_at,
                created_at: now(),
                metadata: JSON.stringify(metadata || null)
              };
              debug('saving invoice:', invoice);
              _context.next = 23;
              return db('invoice').insert(invoice);

            case 23:
              if (!webhook) {
                _context.next = 26;
                break;
              }

              _context.next = 26;
              return addHook(id, webhook);

            case 26:
              return _context.abrupt("return", format(invoice));

            case 27:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function newInvoice(_x) {
      return _ref.apply(this, arguments);
    };
  }();

  var listInvoices = function listInvoices(_) {
    return db('invoice').then(function (rows) {
      return rows.map(format);
    });
  };

  var fetchInvoice = function fetchInvoice(id) {
    return db('invoice').where({
      id: id
    }).first().then(function (r) {
      return r && format(r);
    });
  };

  var delInvoice = /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(id, status) {
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return ln.delinvoice(id, status);

            case 2:
              _context2.next = 4;
              return db('invoice').where({
                id: id
              }).del();

            case 4:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function delInvoice(_x2, _x3) {
      return _ref2.apply(this, arguments);
    };
  }();

  var markPaid = function markPaid(id, pay_index, paid_at, msatoshi_received) {
    return db('invoice').where({
      id: id,
      pay_index: null
    }).update({
      pay_index: pay_index,
      paid_at: paid_at,
      msatoshi_received: msatoshi_received
    });
  };

  var getLastPaid = function getLastPaid(_) {
    return db('invoice').max('pay_index as index').first().then(function (r) {
      return r.index;
    });
  };

  var delExpired = function delExpired(ttl) {
    return db('invoice').select('id') // fetch unpaid invoices expired over `ttl` ago
    .where({
      pay_index: null
    }).where('expires_at', '<', now() - ttl) // try fetching the invoices from c-lightning to make sure they're deleted there.
    // invoices that still exists on c-lightning won't be deleted from charge.
    // c-lightning should be configured for automatic cleanup via autocleaninvoice
    .then(function (invs) {
      return Promise.all(invs.map(function (inv) {
        return ln.listinvoices(inv.id).then(function (r) {
          return r.invoices.length ? null : inv.id;
        });
      }));
    }).then(function (ids) {
      return ids.filter(function (id) {
        return id != null;
      });
    }) // delete all expired invoices not found on c-lightning
    .then(function (ids) {
      return ids.length && db('invoice').whereIn('id', ids)["delete"]();
    });
  };

  var addHook = function addHook(invoice_id, url) {
    return db('invoice_webhook').insert({
      invoice_id: invoice_id,
      url: url,
      created_at: now()
    });
  };

  var getHooks = function getHooks(invoice_id) {
    return db('invoice_webhook').where({
      invoice_id: invoice_id
    });
  };

  var logHook = function logHook(id, err, res) {
    return db('invoice_webhook').where({
      id: id
    }).update(!err ? {
      requested_at: now(),
      success: true,
      resp_code: res.status
    } : {
      requested_at: now(),
      success: false,
      resp_error: err
    });
  };

  return {
    newInvoice: newInvoice,
    listInvoices: listInvoices,
    fetchInvoice: fetchInvoice,
    delInvoice: delInvoice,
    getLastPaid: getLastPaid,
    markPaid: markPaid,
    delExpired: delExpired,
    addHook: addHook,
    getHooks: getHooks,
    logHook: logHook
  };
};